import math
import numpy as np

def ratio_top_rest(values_arg, percentage):
    """
    Implements this formula:
    assets/ratio_10_90_formula.png

    Input can be np.array or list of numbers.
    """

    percentage_top = percentage * 0.01
    values = np.array(values_arg)

    values.sort() # sort in-place in ascending order

    n_users = len(values)
    #~ top_percent_users would be = math.ceil(n_users * percentage_top);
    rest_users = math.floor(n_users * (1 - percentage_top));
    edits_rest = values[:rest_users].sum()
    edits_top  = values[rest_users:].sum()

    return (edits_top / edits_rest)


def ratio_top10_rest(values_arg):
    return ratio_top_rest(values_arg, 10)



import numpy as np


def gini(values_arg, n = None):
    """
    Calculates the gini_coeff using the formula shown in assets/gini_formula.png

    Remind that len(values) here corresponds to n+1 there in that formula and remind that idx here corresponds to i-1 in the formula,
    since python lists are zero-indexed and the formula is one-indexed
    this can be confusing.

    @type  values: a Python list or np.array of numbers to calculate gini coefficient on
    """

    if n is None:
        n = len(values_arg)

    values = np.array(values_arg)

    values.sort() # sort in-place in ascending order

    sum_numerator = 0
    sum_denominator = 0
    for idx in range(0, n):
        i = idx + 1 # idx here corresponds to i-1 in the formula, since python lists are zero-indexed
        sum_numerator += (n + 1 - i) * values[idx]
        sum_denominator += values[idx]
    if sum_denominator == 0:
        return np.NaN

    g_coeff = n + 1 - 2*(sum_numerator/sum_denominator)

    return g_coeff


def gini_corrected(values_arg, n = None):
    """
    Calculates the gini_coeff with a correction for small datasets

    @type  values: a Python list or np.array of numbers to calculate gini coefficient on
    """

    if n is None:
        n = len(values_arg)

    # compute gini coefficient
    g_coeff = gini(values_arg, n)

    # Now, apply (Deltas, 2003 correction) for small datasets:
    # (https://doi.org/10.1162/rest.2003.85.1.226)
    g_coeff *= (1.0 / (n - 1))

    return g_coeff



from setuptools import setup

with open('README.md') as file:
    long_description = file.read()

setup(
    name='inequality_coefficients',
    packages=['inequality_coefficients'],
    version='1.2.0',
    description='Coefficients to measure inequality.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Abel 'Akronix' Serrano Juste",
    author_email='akronix5@gmail.com',
    url='https://github.com/Grasia/inequality_coefficients',
    python_requires='>=3',
    install_requires=['numpy'],
    license='MIT',
    keywords='inequality coefficient index gini ratio top',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research'
    ]
)
